/*
* FILE: join.c
* DESCRIPTION:
*   This example demonstrates how to "wait" for thread completions by using
*   the Pthread join routine.  Threads are explicitly created in a joinable
*   state for portability reasons. Use of the pthread_exit status argument is 
*   also shown. Compare to detached.c
* AUTHOR: 8/98 Blaise Barney
* LAST REVISED:  01/30/09
******************************************************************************/
#include <pthread.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#define NUM_THREADS	4

pthread_mutex_t mutexResult;
int * results;
int gSum = 0;

void *adder(void *t)
{
   int i;
   int tid;
   double result=0.0;
   tid = (int)t;
   
   pthread_mutex_lock(&mutexResult);
   int tmp = gSum;
   printf("Thread %d starting...\n",tid);
   results[tid] = tid;
   tmp += tid;
   printf("Thread %d done.\n" ,tid);
   gSum = tmp;
   pthread_mutex_unlock (&mutexResult);
   pthread_exit((void*) t);
}

int main (int argc, char *argv[])
{
   pthread_t thread[NUM_THREADS];
   pthread_attr_t attr;
   int rc;
   long t;
   void *status;
   results = malloc(NUM_THREADS * sizeof(int)); 
   pthread_mutex_init(&mutexResult, NULL);

   /* Initialize and set thread detached attribute */
   pthread_attr_init(&attr);
   pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);

   for(t=0; t<NUM_THREADS; t++) {
      printf("Main: creating thread %ld\n", t);
      rc = pthread_create(&thread[t], &attr, adder, (void *)t); 
      if (rc) {
         printf("ERROR; return code from pthread_create() is %d\n", rc);
         exit(-1);
         }
      }

   /* Free attribute and wait for the other threads */
   pthread_attr_destroy(&attr);
   int sum = 0;
   for(t=0; t<NUM_THREADS; t++) {
      rc = pthread_join(thread[t], &status);
      if (rc) {
         printf("ERROR; return code from pthread_join() is %d\n", rc);
         exit(-1);
         }
      sum += results[t];
      printf("Main: completed join with thread %ld having a status of %ld\n",t,(long)status);
      }
 
printf("Main: program completed %d : %d. Exiting.\n", gSum, sum);
pthread_exit(NULL);
}
